FUNC_ORDER = [
    "do_fetch",
    "do_unpack",
    "do_patch",
    "do_configure",
    "do_compile",
    "do_install",
    "do_populate_sysroot",
    "do_build",
    "do_package"
]

KNOWN_FUNCS = [
    "do_addto_recipe_sysroot",
    "do_allpackagedata",
    "do_ar_configured",
    "do_ar_original",
    "do_ar_patched",
    "do_ar_recipe",
    "do_assemble_fitimage",
    "do_assemble_fitimage_initramfs",
    "do_bootimg",
    "do_build",
    "do_build_native_sysroot",
    "do_build_sysroot",
    "do_build_target_sysroot",
    "do_build_without_rm_work",
    "do_bundle_files",
    "do_bundle_initramfs",
    "do_ccmake",
    "do_ccmake_diffconfig",
    "do_check",
    "do_checkuri",
    "do_clean",
    "do_cleanall",
    "do_cleanccache",
    "do_cleansstate",
    "do_collect_packagedata",
    "do_compile",
    "do_compile_kernelmodules",
    "do_compile_ptest_base",
    "do_configure",
    "do_configure_ptest_base",
    "do_convert_crlf_to_lf",
    "do_create_extlinux_config",
    "do_create_manifest",
    "do_create_rdepends_inc",
    "do_cve_check",
    "do_deploy",
    "do_deploy_archives",
    "do_deploy_archives_setscene",
    "do_deploy_setscene",
    "do_deploy_source_date_epoch",
    "do_deploy_source_date_epoch_setscene",
    "do_devpyshell",
    "do_devshell",
    "do_diffconfig",
    "do_dumpdata",
    "do_extra_symlinks",
    "do_fetch",
    "do_gcc_stash_builddir",
    "do_gcc_stash_builddir_setscene",
    "do_generate_content",
    "do_generate_toolchain_file",
    "do_get_public_keys",
    "do_image",
    "do_image_complete",
    "do_image_complete_setscene",
    "do_image_qa",
    "do_image_qa_setscene",
    "do_install",
    "do_install_ptest_base",
    "do_install_ptest_perl",
    "do_kernel_checkout",
    "do_kernel_configcheck",
    "do_kernel_configme",
    "do_kernel_link_images",
    "do_kernel_metadata",
    "do_kernel_version_sanity_check",
    "do_lint",
    "do_listtasks",
    "do_locked_sigs",
    "do_menuconfig",
    "do_mkimage",
    "do_multilib_install",
    "do_package",
    "do_package_index",
    "do_package_qa",
    "do_package_qa_setscene",
    "do_package_setscene",
    "do_package_write_deb",
    "do_package_write_deb_setscene",
    "do_package_write_ipk",
    "do_package_write_ipk_setscene",
    "do_package_write_rpm",
    "do_package_write_rpm_setscene",
    "do_package_write_tar",
    "do_packagedata",
    "do_packagedata_setscene",
    "do_patch",
    "do_populate_cve_db",
    "do_populate_ide_support",
    "do_populate_lic",
    "do_populate_lic_deploy",
    "do_populate_lic_setscene",
    "do_populate_sdk",
    "do_populate_sdk_ext",
    "do_populate_sysroot",
    "do_populate_sysroot_setscene",
    "do_preconfigure",
    "do_prepare_copyleft_sources",
    "do_prepare_recipe_sysroot",
    "do_recipe_sanity",
    "do_recipe_sanity_all",
    "do_rm_work",
    "do_rm_work_all",
    "do_rootfs",
    "do_rootfs_wicenv",
    "do_savedefconfig",
    "do_sdk_depends",
    "do_shared_workdir",
    "do_shared_workdir_setscene",
    "do_sizecheck",
    "do_spdx",
    "do_stash_locale",
    "do_stash_locale_setscene",
    "do_strip",
    "do_symlink_kernsrc",
    "do_testexport",
    "do_testimage",
    "do_testsdk",
    "do_testsdkext",
    "do_unpack",
    "do_unpack_and_patch",
    "do_validate_branches",
    "do_warn_musl",
    "do_write_config",
    "do_write_qemuboot_conf"
]
